import rox
from rox import g
import gobject
import os, sys, signal
from popen2 import Popen4
from rox.options import Option

device = Option('device','0,0,0')
speed = Option('speed','8')
simulate = Option('simulate','1')
eject = Option('eject','0')
overburn = Option('overburn','0')
driver = Option('driver','')
buffercount = Option('buffercount','32')

class Burner(rox.Window):
	def __init__(self,tmpfile,filename,maxtrack):
		rox.Window.__init__(self)
		self.maxtrack = maxtrack
		self.line=''
		self.tmpfile=tmpfile
		self.aborting=False
		self.l=g.TextView()
		self.l.set_editable(False)
		self.l.set_wrap_mode(g.WRAP_WORD)
		self.progress = g.ProgressBar()
		self.progress.set_orientation(g.PROGRESS_LEFT_TO_RIGHT)
		swin = g.ScrolledWindow()
		swin.set_policy(g.POLICY_AUTOMATIC, g.POLICY_AUTOMATIC)
		swin.set_shadow_type(g.SHADOW_IN)
		swin.add(self.l)
#		swin.set_border_width(4)

		self.abort=rox.ButtonMixed(g.STOCK_NO,'Abort')
		self.abort.connect('clicked',self.abort_click)
		self.close=rox.ButtonMixed(g.STOCK_CLOSE,'Close')
		self.close.connect('clicked',self.close_click)
		self.close.set_sensitive(False)
		
		box2 = g.HBox(spacing=4)
#		box2.set_border_width(4)
		box2.pack_start(self.progress)
		box2.pack_end(self.close,False)
		box2.pack_end(self.abort,False)

		box = g.VBox(spacing=4)
		box.pack_start(swin)
		box.pack_end(box2,False)
		
		self.add(box)
		self.set_default_size(400,200)
		self.set_border_width(4)
		self.connect("delete-event",self.close_handler)
		if filename:
			t='Burning '+os.path.basename(filename)+' to CDR'
		else:
			t='Burning unsaved TOC to CDR'
		if simulate.int_value: t='Fake'+t
		self.set_title(t)

		self.watch=None
	
		cmd='cdrdao write '
		cmd+='--device '+device.value+' '
		if driver.value: cmd+='--driver '+driver.value+' '
		if simulate.int_value: cmd+='--simulate '
		if speed.int_value: cmd+='--speed '+str(speed.int_value)+' '
		cmd+='--buffers '+str(buffercount.int_value)+' '
		if overburn.int_value: cmd+='--overburn '
		if eject.int_value: cmd+='--eject '
#		cmd+='-n "'+filename+'"'
		cmd+='-n "'+self.tmpfile.name+'"'
		
#		rox.info(cmd)

		self.l.get_buffer().connect('changed',self.auto_scroll)
		
#		os.chdir(os.path.dirname(filename)) #not needed since tmp-tocfile have full pathnames..
		self.burner=Popen4(cmd)
		self.fo=self.burner.fromchild

		self.watch = gobject.io_add_watch(self.fo,gobject.IO_IN | gobject.IO_HUP | gobject.IO_PRI,self.logwatch)
		
		self.show_all()
		
	def auto_scroll(self,buf):
		self.l.scroll_mark_onscreen(buf.get_insert())
		
	def logwatch(self,fo,x):
		buf=self.l.get_buffer()
		c = fo.read(1)
		
		if not c:
			gobject.source_remove(self.watch)

		if self.burner.poll() > -1:
			self.close.set_sensitive(True)
			self.abort.set_sensitive(False)
			
		if self.burner.poll() > 0:
			self.progress.set_text('Burnprocess aborted')
			self.progress.set_fraction(1)
#			return True
	
		if self.burner.poll() == 0:
			self.progress.set_text('Burn finished successfully')
		
		if c == '\n' or c == '\r':
			line = self.line
			self.line = ''
			buf.insert(buf.get_end_iter(),line+'\n')
			if not self.aborting:

				for s in [ 'Starting', 'Writing lead-in', 'Writing lead-out', 'Flushing', 'finished' ]:
					if line.find(s) > -1:
						self.progress.set_text(line)
				if line.find('Writing track') > -1:
					self.progress.set_text(line[0:line.find('(')]+'of '+str(self.maxtrack).zfill(2))
				
			if line.find('Wrote') > -1 and line.find('of') > -1:
				fields=line.split()
				x=float(fields[1])
				y=float(fields[3])
				self.progress.set_fraction(x/y)
		else:
			self.line+=c
		

		return True
		
	def kill_burner(self):
		self.aborting=True
		self.progress.set_text('Waiting for burnprocess to terminate...')
		os.kill(self.burner.pid,signal.SIGQUIT)
#		self.abort.set_sensitive(False)

	def abort_click(self,button):
		if self.burner.poll() == -1:
			self.kill_burner()
			
	def close_click(self,button):
		self.close_handler(None)
		
	def close_handler(self,wid,ev=None):
		if self.burner.poll() == -1:
			self.kill_burner()
			return True
		gobject.source_remove(self.watch)
		self.fo.close()
		self.tmpfile.close()
		if not ev: self.destroy()
		return False
