import rox
from rox import g
import os, time, sys
from rox.loading import *
from rox.saving import *
from rox import get_local_path
from rox.options import Option
import burner
import wave
import tempfile
HAVE_AO = True
try: import ao
except: HAVE_AO = False

app_name = 'RoxDAO'
filename = None
tocfile = None

TYPE_URILIST = 0
TYPE_TRACK = 1
TYPE_XTRACK = 2

PLAY_IMG = rox.app_dir+'/play.png'
PLAY2_IMG = rox.app_dir+'/play2.png'
STOP_IMG = rox.app_dir+'/stop.png'

mp3ogg2wav_path = Option('decoder_path','')

ao_driver = Option('ao_driver','oss')

def samp2msf(x):
	f=int(x/588)
	s=int(f/75)
	m=int(s/60)
	return (m,s%60,f%75)

def samp2msf_str(x):
	return '%02d:%02d.%02d' % (samp2msf(x))

#0, g.TARGET_SAME_APP, g.TARGET_SAME_WIDGET

target_to_track = [
	("application/x-roxdao-track", g.TARGET_SAME_APP, TYPE_TRACK),
	("application/x-roxdao-xtrack", 0, TYPE_XTRACK),
	("text/uri-list", 0, TYPE_URILIST)
]
target_from_track = [
	("application/x-roxdao-track", g.TARGET_SAME_APP, TYPE_TRACK),
	("application/x-roxdao-xtrack", 0, TYPE_XTRACK)
]

default_silence='2'
def silence_dialog():
	global default_silence
#	w=rox.Dialog('Silence',None,g.DIALOG_MODAL,(g.STOCK_OK,1))
	w=rox.Dialog('Silence',None,g.DIALOG_MODAL)
	w.add_button(g.STOCK_CANCEL,g.RESPONSE_CANCEL)
	w.add_button(g.STOCK_OK,g.RESPONSE_OK)
	entry = g.Entry()
	entry.set_text(default_silence)
	w.vbox.add(g.Label('Length of silence in seconds:'))
	w.vbox.add(entry)
	w.show_all()
	r = w.run()
	l = entry.get_text()
	w.destroy()
	if r == g.RESPONSE_OK:
		try: x = int(float(l)*44100)
		except: x = 0
	else:
		x = 0
	return x

class TrackDropArea:
	def __init__(self):
		self.drag_dest_set(
			g.DEST_DEFAULT_MOTION |
			g.DEST_DEFAULT_HIGHLIGHT |
			g.DEST_DEFAULT_DROP,
			target_to_track,
			g.gdk.ACTION_COPY)
		self.connect("drag_data_received", self.receiveCallback)

	def receiveCallback(self, widget, context, x, y, selection, targetType, time):
		tb=self.trackbox
		t=None
		if isinstance(widget, Track):
			pos=tb.tracks.child_get_property(widget,'position')
		else:
			pos=None
		if targetType == TYPE_TRACK:
			if selection.data == "silence":
				t = tb.add_track(None,0,silence_dialog())
				t.trackinc=False
			else:
				t = tb.tracks.get_children()[int(selection.data)]

			if pos != None:
				tb.tracks.reorder_child(t,pos)
				tb.modified=True
				
		elif targetType == TYPE_URILIST:
			t = tb.load_files(extract_uris(selection.data),pos)
						
		elif targetType == TYPE_XTRACK:
			(file,offset,length,trackinc) = selection.data.split(';')
			t = tb.add_track(file,int(offset),int(length))
			if trackinc == 'True': t.trackinc = True
			else: t.trackinc = False
			if pos != None:
				tb.tracks.reorder_child(t,pos)
		tb.track_select(t)
		tb.update_trackmarks()

class DropVBox(g.VBox, TrackDropArea):
	def __init__(self,tb):
		g.VBox.__init__(self)
		TrackDropArea.__init__(self)
		self.trackbox=tb
		
class NewButton(rox.ButtonMixed, XDSLoader):
	def __init__(self):
		rox.ButtonMixed.__init__(self,g.STOCK_NEW,'')
		XDSLoader.__init__(self, ['text/uri-list'])
		self.connect('clicked',self.click)
		self.set_relief(g.RELIEF_NONE)

	def xds_load_uris(self, uris):
		files = ''
		for uri in uris:
			path = get_local_path(uri)
			if path:
				files += ' "'+path+'"'
		os.system(sys.argv[0]+' '+files+'&')
		
	def click(self,b):
		os.system(sys.argv[0]+'&')

class Track(g.Button, TrackDropArea):
	def __init__(self,trackbox,path,offset=0,length=None):
		g.Button.__init__(self)
		TrackDropArea.__init__(self)

		if path: ## track
			if not os.path.isabs(path):
				path = os.path.normpath(os.path.join(os.path.dirname(tocfile),path))

			if not os.path.isfile(path):
				rox.alert('File not found:\n%s' % (path))
				raise IOError
				
			ext = os.path.splitext(path)[1].lower()
			if ext == '.wav':
				f = wave.open(path,'r')
				(chan, width, rate, nframes, ctyp, cnam) = f.getparams()
				f.close()
				self.maxlength = nframes
				if not (chan==2 and width==2 and rate==44100):
					rox.alert('%s is not 16bit 44.1KHz Stereo, and will not work with cdrdao' % (path))
					raise IOError
			else:
				self.maxlength = os.path.getsize(path)
			
			if not length: self.length=self.maxlength-offset
			else: self.length = length
			self.file=path
			self.label=g.Label(os.path.basename(self.file))
		else: ## silence
#			self.filesize = 0
			self.maxlength = 0
#			if not length: rox.croak('silence must have length')
			self.length = length
			self.file=''
			self.label=g.Label()

		if not self.length: raise IOError
			
		self.trackbox=trackbox
		self.offset=offset

		self.upd_size()

		self.connect('size-allocate', self.expose)
		self.add(self.label)
		self.label.show()
		self.trackinc=True
		self.trackmark=g.Label('')
		self.trackmark.hide()
		self.trackbox.layout.put(self.trackmark,-100,0)
		
		if path: ## track
			self.modify_bg(g.STATE_NORMAL,g.gdk.color_parse('#77aa88'))
			self.modify_bg(g.STATE_PRELIGHT,g.gdk.color_parse('#88c099'))
		else: ## silence
			self.modify_bg(g.STATE_NORMAL,g.gdk.color_parse('#778899'))
			self.modify_bg(g.STATE_PRELIGHT,g.gdk.color_parse('#8899b0'))

		self.label.modify_fg(g.STATE_NORMAL,g.gdk.color_parse('#444444'))
		self.label.modify_fg(g.STATE_PRELIGHT,g.gdk.color_parse('#000000'))
		
		self.drag_source_set(
			g.gdk.BUTTON1_MASK,
			target_from_track,
			g.gdk.ACTION_COPY)
		self.connect("drag_data_get", self.sendCallback)

		self.connect('destroy',self.close)

		self.connect('enter-notify-event',self.stop_event)
		self.connect('leave-notify-event',self.stop_event)
		self.connect('focus',self.stop_event)

	def set_length(self,x):
		if x > 0:
			if x+self.offset > self.maxlength and self.maxlength != 0:
				self.length = self.maxlength-self.offset
			else:
				self.length = x

	def set_end(self,x):
		if x > self.offset:
			self.length = x-self.offset
			if x > self.maxlength and self.maxlength != 0:
				self.length = self.maxlength-self.offset
		
	def set_start(self,x):
		newend = self.offset+self.length
		self.offset=x
		self.set_end(newend)

	def stop_event(self,x,y=None):
		return True	
		
	def upd_size(self):
		self.set_size_request(int(self.length/(44100*4*self.trackbox.zoom)),30)
		
	def close(self,x):
		self.trackmark.destroy()
		return True

	def sendCallback(self, widget, context, selection, targetType, eventTime):
#		print >>sys.stderr,selection.target,targetType
		if targetType == TYPE_TRACK:
			pos=self.trackbox.tracks.child_get_property(widget,'position')
			selection.set(selection.target, 8, str(pos))
		elif targetType == TYPE_XTRACK:
			selection.set(selection.target, 8,
				widget.file+';'+
				str(widget.offset)+';'+
				str(widget.length)+';'+
				str(widget.trackinc))

	def toggle_trackinc(self):
		self.trackinc = not self.trackinc
		self.trackbox.modified = True

	def expose(self,t=None,ev=None):
		x = self.get_allocation()[0]
		self.trackbox.layout.move(self.trackmark,x,0)
		

class SilenceButton(g.Button):
	def __init__(self,trackbox):
		g.Button.__init__(self,'Silence')
		self.connect("drag_data_get", self.sendCallback)
		self.drag_source_set(g.gdk.BUTTON1_MASK, target_from_track, g.gdk.ACTION_COPY)
		self.trackbox=trackbox
		self.connect("clicked",self.click_handler)
		
	def click_handler(self,b):
		self.trackbox.add_track(None,0,silence_dialog())

	def sendCallback(self, w, c, selection, type, time):
#		tb=self.trackbox
#		pos=tb.tracks.child_get_property(widget,'position')
		selection.set(selection.target, 8, "silence")

class TocSaveBox(SaveBox, Saveable):
	def __init__(self, roxdao, name, cb, discard):
		SaveBox.__init__(self, self, name, 'text/x-cdrdao-toc', discard=discard)
		self.callback = cb
		self.filename = ''
		self.roxdao = roxdao
	
	def discard(self):
		self.roxdao.destroy()
	
	def save_to_stream(self, stream):
		stream.write(self.roxdao.make_toc(self.filename))

	def save_to_file(self,path):
		self.filename = path
		Saveable.save_to_file(self,path)

	def set_uri(self, uri):
		self.callback(uri)
		self.roxdao.trackbox.modified = False
		Saveable.set_uri(self,uri)

class TrackBox(g.ScrolledWindow):
	def __init__(self,tracksel_callback,roxdao):
		g.ScrolledWindow.__init__(self)
		self.sel_track=None
		self.zoom=0.44
		self.cursor_pos=0
		self.track_pos=0
		self.tracksel_callback=tracksel_callback
		self.roxdao=roxdao
		self.total_length=0
		self.ntracks=0
		self.nblocks=0
		self.modified=False

		self.box=DropVBox(self)
		self.tracks=g.HBox(False)
		self.layout=g.Layout()
		self.tooltips=g.Tooltips()
		
		self.cursor=g.Arrow(g.ARROW_DOWN,g.SHADOW_OUT)
		
		self.playcursor=g.Arrow(g.ARROW_DOWN,g.SHADOW_OUT)
		self.playcursor.modify_fg(g.STATE_NORMAL,g.gdk.color_parse('#0000ff'))
		
		self.layout.set_size_request(-1,20)
		self.layout.modify_bg(g.STATE_NORMAL,g.gdk.color_parse('#bbbbbb'))
		self.layout.put(self.cursor,-10,10)
		self.move_cursor(0)
		
		self.layout.put(self.playcursor,-10,10)
		
		self.layout.connect('button-press-event',self.layout_press)
                
		self.box.set_size_request(-1,66) #62
		self.box.set_border_width(1)
		self.box.pack_start(self.layout,False,False)
		self.box.pack_start(self.tracks,False,False)
		
		self.timeline=None

		self.connect('scroll-event',self.scroll)

		self.set_shadow_type(g.SHADOW_NONE)
		self.set_size_request(200,-1)
		self.set_policy(g.POLICY_ALWAYS,g.POLICY_NEVER)
		self.set_border_width(4)
		
		self.add_with_viewport(self.box)
		
		self.connect('map',self.mapped)
		self.draw_timeline()

	def scroll(self,wid,ev):
		if ev.direction == g.gdk.SCROLL_UP:
			self.zoom_in()
		elif ev.direction == g.gdk.SCROLL_DOWN:
			self.zoom_out()
			
	def draw_timeline(self):
		totlen=0
		tl=g.Layout()
		tl.modify_bg(g.STATE_NORMAL,g.gdk.color_parse('#f0f0f0'))
		tl.set_size_request(-1,14)
		step = 60*self.zoom
		z=44100*4*step
		totlen = int(1+self.total_length/z)
		
		for x in range(0,totlen):
			(m,s,f) = samp2msf(x*z)
			l = g.Label("|%02d:%02d" % (m, s))
			l.show()
			tl.put(l,int(x*60),0)
		
		if self.timeline:
			self.timeline.destroy()

		self.timeline=tl
		self.box.pack_end(self.timeline,False,False)
		self.timeline.show()
		

	def update_trackmarks(self):
		i=0
		x=0
		self.total_length=0
		for t in self.tracks.get_children():
			x+=1
			self.total_length+=t.length
			if t.trackinc:
				t.trackmark.show()
				i+=1
			else:
				t.trackmark.hide()
			t.trackmark.set_text(str(i))
		self.ntracks=i
		self.nblocks=x
		self.tracksel_callback(self.sel_track,None)
		
	def mapped(self,widget=None): 
		self.layout.window.set_cursor(g.gdk.Cursor(g.gdk.SB_DOWN_ARROW))

	def add_track(self,path,offset=0,length=None):
		try: t=Track(self,path,offset,length)
		except:
			return None
		if not path: path="(Silence)"
		self.tooltips.set_tip(t,path)
		self.tracks.pack_start(t,False,False)
		t.connect("button-press-event",self.track_click)
		t.connect("focus-in-event",self.tracksel_callback)
		self.track_select(t)
		t.show()
		self.update_trackmarks()
		self.draw_timeline()
		self.modified = True
		return t
		
	def remove_track(self,button):
		if not self.sel_track: return
		self.sel_track.destroy()
		self.sel_track=None
		self.update_trackmarks()
		self.modified = True

	def split_track(self,button):
		self.track_select(self.track_at_pos(self.cursor_pos)[0])
		if not self.sel_track: return
		t1 = self.sel_track
		if not self.track_pos > 0: return
		t2 = self.add_track(t1.file,self.track_pos+t1.offset,t1.length-self.track_pos)
		t1.length = self.track_pos
		t1.upd_size()
		
		pos=self.tracks.child_get_property(t1,'position')
		self.tracks.reorder_child(t2,pos+1)
		self.update_trackmarks()
		self.modified = True
		
	def merge_tracks(self,button):
		if not self.sel_track: return
		t1 = self.sel_track
		pos = self.tracks.child_get_property(t1,'position')
		try: t2 = self.tracks.get_children()[pos+1]
		except: rox.alert("No block to merge..."); return
		if t1.file != t2.file: rox.alert("The two blocks must be same file"); return
#		t1.length += t2.length
#		if t1.maxlength and t1.length > t1.maxlength: t1.length = t1.maxlength
		t1.set_length(t1.length+t2.length)
		t1.upd_size()
		t2.destroy()
		self.update_trackmarks()
		self.modified = True

	def move_cursor(self,pos):
		(w,h)=self.cursor.size_request()
		x=int(pos-(w/2))
		self.layout.move(self.cursor,x,10)
		self.cursor_pos=pos

		
	def track_select(self,t):
		if self.sel_track: self.sel_track.set_state(g.STATE_NORMAL)
		self.sel_track=t
		if t:
			#t.grab_focus()
			t.set_state(g.STATE_PRELIGHT)
		self.tracksel_callback(t,None)
		
	def track_click(self,t,ev):
		self.track_select(t)
		if ev.button == 2:
			a=t.get_allocation()
			self.move_cursor(a.x)
			self.track_pos=0
			self.tracksel_callback(t,None)
		elif ev.button == 3:
			t.toggle_trackinc()
			self.update_trackmarks()
		return True

	def center_cursor(self):
		a=self.get_allocation()
		x=a.width/2
		self.get_hadjustment().set_value(self.cursor_pos-x)
		
	def upd_zoom(self,zoom):
		x=self.cursor_pos*self.zoom
		self.move_cursor(x/zoom)
		
		self.zoom=zoom
		for t in self.tracks.get_children():
			t.upd_size()
		self.draw_timeline()
		
		self.center_cursor()

	def zoom_in(self,button=None):
		if self.zoom > 0.02:
			self.upd_zoom(self.zoom/1.5)
		
	def zoom_out(self,button=None):
		if self.zoom < 2:
			self.upd_zoom(self.zoom*1.5)
			
	def track_at_pos(self,x):
		size = 0
		for t in self.tracks.get_children():
			(w,h) = t.size_request()
			size += w
			if size > x:
				a=t.get_allocation()
				pos=int((x-a.x)*(44100*4*self.zoom))
#				self.track_select(t)
				return (t, pos)
		return (None, None)
	
	def layout_press(self,widget,ev):
		if ev.button==3:
			self.center_cursor()
			return
		self.move_cursor(ev.x)
		(t, pos) = self.track_at_pos(ev.x)
		if pos:
			self.track_pos = pos
		self.track_select(t)

	def load_files(self,files,pos=None):
		list = []
		conv = ''
		t = None
		for fn in files:
			file = rox.get_local_path(fn)
			if file:
				(base, ext) = os.path.splitext(file)
				ext = ext.lower()
				if ext == '.mp3' or ext == '.ogg':
					conv += ' "'+file+'"'
					list.append(base + '.wav')
				else:
					list.append(file)
			else:
				print >>sys.stderr, '%s\nis not an existing local file' % (fn)

		if conv and rox.confirm('Some files need to be converted to .wav',g.STOCK_EXECUTE,'Convert'):
			cmd = mp3ogg2wav_path.value or rox.app_dir+'/../Mp3Ogg2Wav/AppRun'
			os.system(cmd+conv)
			
		for file in list:
			ext = os.path.splitext(file)[1].lower()
			if ext == '.toc':
				t = self.load_toc(file,pos)
			else:
				t = self.add_track(file)
				if pos != None:
					self.tracks.reorder_child(t,pos)
					pos+=1
		return t

	def load_toc(self,path,pos=None):
		global tocfile
		trackinc=False
		f = file(path,"rU")
		tocfile=path

		t = None
		for line in f:
			line=line.strip()
			if line == "TRACK AUDIO": trackinc=True
			x = ""
			if len(line) > 1:
				x = line.split()[0]

			if x == "FILE" or x == "AUDIOFILE":
				fields = line.split()[1:]
				path = fields[0].strip('"')
				path = os.path.normpath(path)
				
				offset = get_time(fields[1])
				if len(fields)>2: length = get_time(fields[2])
				else: length = None
				
				try:
					t = self.add_track(path,offset,length)
					if trackinc: t.trackinc=True
					else: t.trackinc=False
					if pos != None:
						self.tracks.reorder_child(t,pos)
						pos+=1

				except:
					print >>sys.stderr, "load_toc() failed on %s" % path
#					rox.report_exception()
				trackinc=False
				
			elif x == "SILENCE":
				length = get_time(line.split()[1])
				t = self.add_track(None,0,length)
				if trackinc: t.trackinc=True
				else: t.trackinc=False
				if pos != None:
					self.tracks.reorder_child(t,pos)
					pos+=1

				trackinc=False
				
		f.close()
		if not filename:
			self.modified = False
			self.roxdao.update_title(tocfile)
		return t


class TrackInfo(g.VBox):
	def __init__(self):
		g.VBox.__init__(self)
		self.file=self.mkinfo()
		self.pos=self.mkentry()
		self.pos.set_editable(False)
		self.maxlength=self.mkentry()
		self.maxlength.set_editable(False)
		self.length=self.mkentry()
		self.start=self.mkentry()
		self.end=self.mkentry()
		self.trackinc=g.CheckButton('xx')
		self.trackinc.connect('toggled',self.trackinc_toggle)
		
		for (wid, hand) in [
				(self.length, Track.set_length),
				(self.start, Track.set_start),
				(self.end, Track.set_end) ]:
			wid.connect('changed',self.entry_changed,hand)
			wid.connect('activate',self.entry_activate)
		
#		self.length.connect('changed',self.entry_changed,Track.set_length)
#		self.start.connect('changed',self.entry_changed,Track.set_start)
#		self.end.connect('changed',self.entry_changed,Track.set_end)
		
		self.tab=g.Table(6,2,False)
		self.tab.set_col_spacings(4)
		self.tab.set_row_spacings(2)
		self.tab.set_border_width(4)
		
		self.internal_change=False
		self.track=None
		self.upd_entries=False
		
		l=self.mklab('File: ')
		box=g.HBox()
		box.pack_start(l,False)
		box.pack_start(self.file,False)
		box.set_border_width(4)
		self.add(box)
		
		self.mklab('TrkMark:',0,0)
		self.putwid(self.trackinc,1,0)
		self.mklab('Start:',2,0)
		self.putwid(self.start,3,0)
		self.mklab('FileLen:',4,0)
		self.putwid(self.maxlength,5,0)
		
		self.mklab('Cursor:',0,1)
		self.putwid(self.pos,1,1)
		self.mklab('End:',2,1)
		self.putwid(self.end,3,1)
		self.mklab('Length:',4,1)
		self.putwid(self.length,5,1)
		
#		self.mod=g.Label('')
		self.add(self.tab)
#		self.add(self.mod)
	
	def putwid(self,wid,col,row):
		self.tab.attach(wid,col,col+1,row,row+1,g.FILL)
		
	def mkinfo(self):
		l=g.Label('')
		l.set_alignment(0,0.5)
		return l

	def mkentry(self):
		e=g.Entry()
		e.set_width_chars(9)
#		e.set_has_frame(False)
		return e

	def mklab(self,txt,col=None,row=None):
		l=g.Label('')
		l.set_alignment(1,0.5)		
		l.set_markup('<b>'+txt+'</b>')
		if row != None: self.tab.attach(l,col,col+1,row,row+1,g.FILL)
		return l
		
	def trackinc_toggle(self,wid):
		if self.track and not self.internal_change:
			self.track.trackinc=wid.get_active()
			self.track.trackbox.update_trackmarks()
			self.track.trackbox.modified = True

	def entry_activate(self,wid):
		if not self.upd_entries: return
		self.upd_entries = False
		self.update(self.track)

	def entry_changed(self,wid,x):
		if self.internal_change or not self.track: return
		try: time = get_time(wid.get_text())
		except: return
#		setattr(self.track,x,time)
		x(self.track,time)
		self.track.upd_size()
#		self.update(self.track)
		self.track.trackbox.modified = True
		self.upd_entries = True

	def update(self,t):
		self.track=t
		if t:
#			if t.trackbox.modified: self.mod.set_text('*')
#			else: self.mod.set_text('')
			for w in self.get_children():
				w.set_sensitive(True)

			self.internal_change=True
			if t.file: self.file.set_text(t.file)
			else: self.file.set_text('(Silence)')
			self.trackinc.set_active(t.trackinc)
			self.trackinc.set_label(t.trackmark.get_text().zfill(2))
			self.pos.set_text(samp2msf_str(t.trackbox.track_pos))
			self.length.set_text(samp2msf_str(t.length))

			self.maxlength.set_text(samp2msf_str(t.maxlength))
#			self.maxlength.set_text(str(t.maxlength))

			self.start.set_text(samp2msf_str(t.offset))
			self.end.set_text(samp2msf_str(t.offset+t.length))
			self.internal_change=False
##TODO: total cursor			
#			print>>sys.stderr,samp2msf_str(t.trackbox.cursor_pos*4*44100*t.trackbox.zoom)
		else:
			for w in self.get_children():
				w.set_sensitive(False)
			x='xx:xx.xx'
			self.file.set_text('(no block selected)')
			self.trackinc.set_active(False)
			self.trackinc.set_label('xx')
			for w in (self.pos, self.length, self.start, self.maxlength, self.end):
				w.set_text(x)

def quick_help(wid):
	w=rox.Window()
	w.set_title(app_name+" Quick Help")
	
	t=g.TextView()
	t.set_cursor_visible(False)
	t.set_editable(False)
	t.set_border_width(4)
	t.get_buffer().set_text(
"""Drag .wav or raw audio files from a filer window
and drop them on the main area to add them.
You can move the order by dragging them around.
Drop .toc files to append them to your project.

left-click on a block to select it,
right-click to toggle trackmark on/off,
middle-click to set cursor to start of block.

click in the area above the blocks to move the cursor,
right-click to center the viewport around the cursor.

The scroll-wheel zooms in/out.

It's possible to drag blocks between RoxDAO applications.""")
	w.add(t)
	w.show_all()

class Space(g.EventBox):
	def __init__(self,x=12):
		g.EventBox.__init__(self)
		self.set_size_request(x,20)
		self.add(g.VSeparator())

def get_time(s):
	s=s.replace('.',':')
	if s.find(':') > -1:
		(m,s,f)=s.split(':')
		x=int(f)*588
		x+=int(s)*44100
		x+=int(m)*44100*60
	else:
		x=int(s)
	return x

class RoxDao(rox.Window):
	def __init__(self,fn=None):
		rox.Window.__init__(self)
		self.set_default_size(g.gdk.screen_width() * 2 / 3,-1)
		self.trackbox = TrackBox(self.upd_info,self)
		self.track_info=TrackInfo()

#		info = g.HBox()	
		frame = g.Frame(' Selected Block ')
		frame.add(self.track_info)
		frame.set_shadow_type(g.SHADOW_IN)
		frame.set_border_width(4)
#		info.add(frame)
		info=frame
		
		bar = g.Toolbar()
		bar.set_style(g.TOOLBAR_ICONS)
		
		bar.insert_stock(g.STOCK_CLOSE,'Quit '+app_name,None,self.close,None,-1)
		bar.insert_widget(NewButton(),'Open new '+app_name+' window',None,-1)
		bar.insert_stock(g.STOCK_SAVE,'Save TOC',None,self.save_toc,None,-1)
		bar.insert_stock(g.STOCK_CDROM,'Burn to CD',None,self.burn_cd,None,-1)
		bar.insert_stock(g.STOCK_PREFERENCES,'Settings',None,self.show_settings,None,-1)
		bar.insert_stock(g.STOCK_DIALOG_INFO,'Quick help',None,quick_help,None,-1)
		bar.insert_widget(Space(),None,None,-1)

		bar.insert_stock(g.STOCK_DELETE,'Remove selected block',None,self.trackbox.remove_track,None,-1)
		bar.insert_stock(g.STOCK_CUT,'Split block at cursor',None,self.trackbox.split_track,None,-1)
		bar.insert_stock(g.STOCK_GOTO_LAST,'Merge with next block',None,self.trackbox.merge_tracks,None,-1)
		bar.insert_widget(SilenceButton(self.trackbox),'Drag this to the main area to add silence',None,-1)
		bar.insert_stock(g.STOCK_ZOOM_IN,'Zoom in',None,self.trackbox.zoom_in,None,-1)
		bar.insert_stock(g.STOCK_ZOOM_OUT,'Zoom out',None,self.trackbox.zoom_out,None,-1)
		if HAVE_AO:
			self.playimg = g.Image()
			self.set_playing(False)
			bar.append_item('Play','Play at cursor',None,self.playimg,self.play)
			i = g.Image()
			i.set_from_file(STOP_IMG)
			bar.append_item('Stop','Stop',None,i,self.stop)
			self.playfile = None
			self.playtrack = None
			self.playcursor_pos = 0
			self.playpos = 0
			self.playdev = None
		
		bar.insert_widget(Space(),None,None,-1)
		
		self.length_label=g.Label('')
		self.length_label.set_padding(8,0)
		bar.insert_widget(self.length_label,None,None,-1)

		self.play_label=g.Label('')
		self.play_label.set_padding(8,0)
		bar.insert_widget(self.play_label,None,None,-1)

		box = g.VBox() #mainbox		
		box.pack_start(bar,False)
		box.pack_end(self.trackbox,False)
		box.pack_start(info,False)

		self.add(box)
		self.show_all()
		self.upd_info(None,None)
		self.set_title(app_name+' - <Untitled>')
		self.connect('delete-event',self.close)
		
		
		g.gdk.flush()
		
		if fn: self.trackbox.load_files(fn)

	def move_playcursor(self,pos):
		w=self.trackbox.playcursor.size_request()[0]
		x=int(pos-(w/2))
		self.trackbox.layout.move(self.trackbox.playcursor,x,10)
		self.playcursor_pos=pos
		
	def set_playing(self,x):
		self.playing = x
		if x:
			self.playimg.set_from_file(PLAY2_IMG)
			self.trackbox.playcursor.show()
		else:
			self.playimg.set_from_file(PLAY_IMG)
			self.trackbox.playcursor.hide()

	def stop(self,button):
		self.set_playing(False)

	def play_next(self):
		pos=self.trackbox.tracks.child_get_property(self.playtrack,'position')
		pos+=1
		t = self.trackbox.tracks.get_children()[pos]
#		print >>sys.stderr,t.file
		return t
		
	def play(self,button=False,t=False):
		if not t:
			t = self.trackbox.track_at_pos(self.trackbox.cursor_pos)[0]
			cursor_pos = self.trackbox.cursor_pos
			track_pos = self.trackbox.track_pos
		else:
			cursor_pos = self.playcursor_pos
			track_pos = 0
		if not t: return
		
		offset = t.offset + track_pos
		self.playend = t.offset + t.length
		self.playadjust = (44100*4*self.trackbox.zoom*cursor_pos)-offset

		if self.playtrack is None or self.playtrack.file != t.file:
			self.playtrack = t
			if self.playfile: self.playfile.close()
			if t.file: self.playfile = wave.open(t.file,'r')
			else: self.playfile = None
			
		if self.playfile:
			self.playfile.setpos(offset)
		else:
			self.playpos = 0
			
		if self.playing:
			return
		else:
			self.set_playing(True)

		try:
			self.playdev = ao.AudioDevice(ao_driver.value)
		except:
			rox.alert('Could not open audio device, did you choose the right LibAO driver in settings?')
			self.playdev = None
			
		if self.playdev:
			silence = '\x00'*4*1024
			while self.playing:
				if self.playfile:
					self.playpos = self.playfile.tell()
					buf = self.playfile.readframes(1024)
			
				if self.playpos >= self.playend:
					nt = self.play_next()
					if nt:
						self.play(t=nt)
					else: break
					
				if self.playfile:
					self.playdev.play(buf)
				else:
					self.playdev.play(silence)
					self.playpos += 1024
					
				self.play_label.set_text(samp2msf_str(self.playpos))
				self.move_playcursor((self.playpos+self.playadjust)/(44100*4*self.trackbox.zoom))
				while g.events_pending(): g.main_iteration(False)


		self.set_playing(False)
		if self.playfile: self.playfile.close()
		self.playfile = None
		self.playtrack = None
		self.playdev = None
		self.play_label.set_text('')
		
	def show_settings(self,button):
		rox.edit_options()
		
	def burn_cd(self,button):
		if rox.confirm('Insert an empty CD-R...',g.STOCK_CDROM,'Burn CD'):
			f = tempfile.NamedTemporaryFile(prefix='tmp.RoxDAO.')
			f.write(self.make_toc())
			f.flush()
			burner.Burner(f,filename,self.trackbox.ntracks)
			pass

	def save_toc(self,button=None,discard=False):
		fn = filename or 'NewTOC.toc'
		savebox=TocSaveBox(self,fn,self.update_title,discard)
		savebox.show()

	def make_toc(self,filename=None):
		if filename:
			filelist = [filename]
			for t in self.trackbox.tracks.get_children():
				if t.file: filelist.append(t.file)
			prefix = os.path.commonprefix(filelist)
			prefix = prefix[:prefix.rfind('/')+1]
			if prefix == '/': prefix = None
			
		# dirname of tocfile must be same as all or none of the tracks dirnames
			if os.path.dirname(filename) in map(os.path.dirname,filelist[1:]):
				for fn in filelist[1:]:
					if not os.path.dirname(filename) == os.path.dirname(fn):
						prefix = None
						break
		txt='CD_DA\n'			
		for t in self.trackbox.tracks.get_children():
			if t.trackinc:
				txt+='TRACK AUDIO\n'
			if t.file:
				file = t.file
				if filename and prefix:
					file = t.file[len(prefix):]
				txt+='FILE "'+file+'" '+str(t.offset)+' '+str(t.length)+'\n'
			else:
				txt+='SILENCE '+str(t.length)+'\n'
		return txt

	def update_title(self,title):
		global filename
		filename = title
		self.set_title(app_name+' - '+title)

	def close(self,widget,ev=None):
		self.set_playing(False)
		if self.trackbox.modified:
			self.save_toc(discard=True)
			return True
		elif not ev:
			self.destroy()
		else:
			return False
		
	def upd_info(self,t,ev):
		self.length_label.set_text("%d Tracks (%d Blocks): " % (self.trackbox.ntracks, self.trackbox.nblocks)+samp2msf_str(self.trackbox.total_length))
		self.track_info.update(t)
		